<?php
class MaintenanceJobs{
	
    static $report="/usr/sbin/spxreport";
    static $boot_config="/boot-config";
    
    static function saveSymlink($target, $link) {
        if ( $_ENV['OS']=='Windows_NT' ) {
            if ($target[0]=='/')
                copy( $target, $link );
            else
                copy( dirname($link)."/".$target, $link );
            file_put_contents($link.".link", $target);
            return true;
        }
        if ( ! symlink($target, $link) ) {
            unlink($link);
            if ( ! symlink($target, $link) ) {
                return "Cannot save settings- File: $link as symlink to ($target)";
            }
        }
        exec('sync');
        return true;
    }
    
    static function saveFile($name, $data, $secret=false, $group='raperca', $ignore_empty=true) {
        if ($data=="" && $ignore_empty) return true;

        $fname=tempnam(dirname($name), basename($name));
        if (file_put_contents($fname,$data)===FALSE){
            return 'Cannot save settings- File: ($name).<br/>Data: '.htmlspecialchars($data);
        }else{
            if (!$secret) {
                chmod($fname, 0644);
            } else {
                chmod($fname, $group ? 0640 : 0600);
                if ($group)
                     chgrp($fname, $group);
            }
            exec('sync');
            if ( $_ENV['OS']=='Windows_NT' && file_exists( $name ) ) {
                if ( copy( $fname, $name ) ) {
                    unlink($fname);
                    exec('sync');
                    return true;
                } else {
                    unlink($fname);
                    return "Cannot save settings- File: ($name).<br/>Data: ".htmlspecialchars($data);
                }
            } else if ( rename($fname, $name) ){
                exec('sync');
                return true;
            } else {
                unlink($fname);
                return "Cannot save settings- File: ($name).<br/>Data: ".htmlspecialchars($data);
            }
        }
        return true;
    }
    
    // Copies the given source file or data data to the destination file
    // under the failsafe-data filesystem. If srcdata is FALSE the
    // destination file is removed instead. If isfile is true srcdata is a
    // file name, otherwise it is the data to copy. Returns false if the
    // failsafe-data data could not be updated.
    static function setSafeModeData($dstfile, $srcdata, $isfile = TRUE){
        if ( $_ENV['OS']=='Windows_NT' ) return true;
        $out="";
        $status="";
        if ($srcdata) {
            if ($isfile) {
                $args = "copy " . escapeshellarg($dstfile) . " " . escapeshellarg($srcdata);
            } else {
                $args = "set " . escapeshellarg($dstfile) . " " . escapeshellarg($srcdata);
            }
        } else {
            $args = "remove " . escapeshellarg($dstfile);
        }

        exec("failsafe-data " . $args, $out, $status);
        return ($status == 0);
    } 
    
    
    function getReportFile( $serial, $ext=false) {
        
		header("Content-type: application/x-gzip");
        
        if ( $ext ){
            header('Content-Disposition: attachment; filename="spxreport-ext-'.$serial.'.tar.gz"');
            passthru("/usr/sbin/spxreport -x", $ret);
        } else { 
            header('Content-Disposition: attachment; filename="spxreport-'.$serial.'.tar.gz"');
            passthru("/usr/sbin/spxreport", $ret);
        }
    }
    
    function checkSSH () {
        exec('/etc/init.d/ssh-delayed status', $dummy, $ret);
        return $ret==0;
    }
    
    function manageSSH( $start=true ) {
        if ( $start ) {
            exec('/etc/init.d/ssh-delayed start');
        } else {
           exec('/etc/init.d/ssh-delayed stop'); 
        }
    }
    function resetFactory( ) {
        $actions = "";
        if ( file_exists( self::$boot_config ) )
            $actions = file_get_contents(self::$boot_config);
        $actions .= "reset\n";
        
        return file_put_contents(self::$boot_config, $actions);
    }
    function formatStorage( ) {
        $actions = "";
        if ( file_exists( self::$boot_config) )
            $actions = file_get_contents(self::$boot_config);
        $actions .= "format\n";
        
        return file_put_contents(self::$boot_config, $actions);
    }
    function resetContent( ) {
        exec('/etc/raperca/resetcontent');
    }
    
    function clearLogs( ) {
        exec('rm -rf /srv/raperca/log/*');
        
        $pid = trim(file_get_contents('/var/run/raperca.pid'));
        posix_kill($pid, SIGUSR1 );
    }
    function clearStreaming( ) {
        exec('rm -rf /srv/raperca/capture/*');
    }
    function clearCache( ) {
        if (touch('/var/spool/raperca/clean-cache')) {
            exec('sync'); // make sure the clean-cache file is on stable storage
            return true;
        } else {
            return false;
        }
    }
    function clearNtp( ) {
        return touch('/var/spool/spxutils/ntpreset');
    }
	
	static function shutdown( $msg, $recovery=false, $delay=false ) {
        if ( $msg == "" ) $msg="user initiated shutdown : unknown reason";
		syslog(LOG_NOTICE, $msg );
		
		if ( $recovery ) {
			exec('/etc/init.d/bootcount noreset');
			exec('echo 100 > /proc/bootcount');
		} 
		if ( $delay==false )
			exec('exec shutdown -r now');
		else
			exec("exec shutdown -r " . escapeshellarg($delay) . " &> /dev/null < /dev/null &");
    }
}
